var Map = new (function() {
	
	var self = this;
	this.leaflet = false;
	this.markers = {};
	this.ready = false;
	this.isTouchDevice = false;
	this._savePosTimeout = null;


	this.initLeaflet = function () {

		self.leaflet = L.map('map');
		if (localStorage.lastLat && localStorage.lastLng && localStorage.lastZoom) {
			self.leaflet.setView([ localStorage.lastLat * 1, localStorage.lastLng * 1 ], localStorage.lastZoom * 1);
		}
		else {
			self.leaflet.setView(defaultCoords, 15);
		}

		/*
		var OpenStreetMap_HOT = L.tileLayer('http://{s}.tile.openstreetmap.fr/hot/{z}/{x}/{y}.png', {
			attribution: '&copy; <a href="http://www.openstreetmap.org/copyright">OpenStreetMap</a>, Tiles courtesy of <a href="http://hot.openstreetmap.org/" target="_blank">Humanitarian OpenStreetMap Team</a>'
		}).addTo(self.leaflet);
		var Hydda_Full = L.tileLayer('http://{s}.tile.openstreetmap.se/hydda/full/{z}/{x}/{y}.png', {
			minZoom: 0,
			maxZoom: 18,
			attribution: 'Tiles courtesy of <a href="http://openstreetmap.se/" target="_blank">OpenStreetMap Sweden</a> &mdash; Map data &copy; <a href="http://www.openstreetmap.org/copyright">OpenStreetMap</a>'
		}).addTo(self.leaflet);
		var OpenMapSurfer_Roads = L.tileLayer('http://openmapsurfer.uni-hd.de/tiles/roads/x={x}&y={y}&z={z}', {
			minZoom: 0,
			maxZoom: 20,
			attribution: 'Imagery from <a href="http://giscience.uni-hd.de/">GIScience Research Group @ University of Heidelberg</a> &mdash; Map data &copy; <a href="http://www.openstreetmap.org/copyright">OpenStreetMap</a>'
		}).addTo(self.leaflet);
		*/
		
		var OpenStreetMap_HOT = L.tileLayer('http://{s}.tile.openstreetmap.se/hydda/full/{z}/{x}/{y}.png', {
			minZoom: 0,
			maxZoom: 18,
			attribution: 'Tiles courtesy of <a href="http://openstreetmap.se/" target="_blank">OpenStreetMap Sweden</a> &mdash; Map data &copy; <a href="http://www.openstreetmap.org/copyright">OpenStreetMap</a>'
		}).addTo(self.leaflet);

		self.hereIcon = L.icon({
			iconUrl: 'assets/here_l.png',
			iconSize:     [16, 16],
			iconAnchor:   [8, 8],
		});

		self.leaflet.on('click', function() {
			$(".brInfo").hide();
			$(".brInfoClose").hide();
			$(".infoPanel").hide();
			$(".controls .info").removeClass("active");
		});

		self.leaflet.on('movestart', function() {
			if (self._savePosTimeout) clearTimeout(self._savePosTimeout);
		});
		self.leaflet.on('moveend', self._mapPosChanged);
		self.leaflet.on('zoomlevelschange', self._mapPosChanged);

		self.ready = true;
		$(window).trigger("mapReady");
	}

	this.init = function () {

		self.isTouchDevice = (('ontouchstart' in window) || (navigator.MaxTouchPoints > 0) || (navigator.msMaxTouchPoints > 0));
		if (self.isTouchDevice) {
			$("body").addClass("touch");
		}

		self.initLeaflet();
	}

	this.addMarker = function(id, lat, lng, html) {

		if (self.markers[id]) return;

		var m = L.marker([lat, lng]).addTo(self.leaflet);

		m.id = id;
		m.html = html;

		m.on('click', self._markerCallbackClick);
		m.on('mouseover', self._markerCallbackMouseover);
		m.on('mouseout', self._markerCallbackMouseout);

		self.markers[id] = m;

		return m;
	}

	this._markerCallbackClick = function() {

		if (self.isTouchDevice) {
			self._markerCallbackMouseover.call(this);
		}
		else {
			window.open($(".brInfo").find("h3 a").attr("href")+"/menu#daily");
		}
	}

	this._markerCallbackMouseover = function() {

		var html = this.html
			.replace(/\b([0-9]+[0-9\,\/]* ?(g|l|ml))\b/g, '<span class="weight">\$1</span> ')
			.replace(/([\/\(] ?[0-9, ]+[\/\)])/g, '<span class="weight">\$1</span> ');

		if (self.isTouchDevice) {
			$(".brInfoClose").remove();
			$(".brInfo").show().html(html);
			$("body").append('<a href="#" class="brInfoClose"><i class="fa fa-times"></i></a>');
		}
		else {
			$(".brInfo").show().html(html + '<div class="open">Kliknutím otvoríš detaily na <b>zomato.com</b></div>');
		}
	}

	this._markerCallbackMouseout = function() {

		$(".brInfo").hide();
	}

	this._mapPosChanged = function() {

		if (self._savePosTimeout) clearTimeout(self._savePosTimeout);

		self._savePosTimeout = setTimeout(self._mapPosChangedFun, 2000);
	}

	this._mapPosChangedFun = function() {
		console.log("pos saved");
		var c = self.leaflet.getCenter();
		localStorage.lastLat = c.lat;
		localStorage.lastLng = c.lng;
		localStorage.lastZoom = self.leaflet.getZoom();
	}

	this.geoLocate = function() {

		if (navigator.geolocation) {
			$(".controls a.locate").addClass("active");

			var stop = function() {

				navigator.geolocation.clearWatch( watchID );
				$(".controls a.locate").removeClass("active");
			};

			var timeout;
			var posReceived = false;
			var options = { enableHighAccuracy: true, maximumAge: 30000, timeout: 2000 };
			var watchID = navigator.geolocation.watchPosition( function success(geo) {

				if (self.locationMarker) {
					self.locationMarker.setLatLng([geo.coords.latitude, geo.coords.longitude])
				}
				else {
					self.locationMarker = L.marker([geo.coords.latitude, geo.coords.longitude], {icon: self.hereIcon}).addTo(self.leaflet);
				}

				if (!posReceived) {

					self.leaflet.panTo([geo.coords.latitude, geo.coords.longitude]);
					posReceived = true;
				}

				if (geo.coords.accuracy < 50 /*meters*/) {
					stop();
					clearTimeout(timeout);
				}
			}, function error (err) {

				if (err.code == 3) return; // timeout

				alert("Nedá sa zistiť poloha");

				$(".controls a.locate").removeClass("active");
			}, options );

			timeout = setTimeout( stop, 20000 );
		}
		else {
			alert("Tvoj prehliadač nepodporuje geolokáciu");
		}
	}
})();

$(function() {
	Map.init();

	$("#restaurants > .restaurant").each(function() {
		Map.addMarker($(this).data("id"), $(this).data("lat"), $(this).data("lng"), $(this).html());
	});

	$(".controls .locate").click(function() {
		if ($(this).hasClass("active")) return false;

		Map.geoLocate();
		return false;
	});

	$(".controls .info").click(function() {
		$(this).toggleClass("active");

		if ($(this).hasClass("active")) {
			$(".infoPanel").show();
		}
		else {
			$(".infoPanel").hide();
		}

		return false;
	});

	$(".brInfo").on("click", "a", function() {
		window.open($(this).closest(".brInfo").find("h3 a").attr("href")+"/menu#daily");

		return false;
	});

	$(document).on("click", ".brInfoClose", function() {
		$(".brInfo").hide();
		$(".brInfoClose").hide();

		return false;
	});
	
	if (!Map.isTouchDevice) {
		$(document).mousemove(function(e) {
			var top = e.pageY - ($(".brInfo").height() / 2);
			if (top < 0) top = 0;
			$(".brInfo").css({left: e.pageX+10+"px", top: top + "px"});
		});
	}
});